@@ -27,14 +27,13 @@ typedef enum {
 	SOUP_CONNECTION_NTLM_FAILED
 } SoupConnectionNTLMState;
 
-struct SoupConnectionNTLMPrivate {
-	char *user;
+typedef struct {
 	guchar nt_hash[21], lm_hash[21];
 	SoupConnectionNTLMState state;
-};
+} SoupConnectionNTLMPrivate;
+#define SOUP_CONNECTION_NTLM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_CONNECTION_NTLM, SoupConnectionNTLMPrivate))
 
-#define PARENT_TYPE SOUP_TYPE_CONNECTION
-static SoupConnectionClass *parent_class;
+G_DEFINE_TYPE (SoupConnectionNTLM, soup_connection_ntlm, SOUP_TYPE_CONNECTION)
 
 static char     *soup_ntlm_request         (void);
 static gboolean  soup_ntlm_parse_challenge (const char  *challenge,
@@ -47,58 +46,51 @@ static char     *soup_ntlm_response        (const char  *nonce,
 					    const char  *domain);
 
 static void
-init (GObject *object)
+soup_connection_ntlm_init (SoupConnectionNTLM *ntlm)
 {
-	SoupConnectionNTLM *ntlm = SOUP_CONNECTION_NTLM (object);
-
-	ntlm->priv = g_new0 (SoupConnectionNTLMPrivate, 1);
 }
 
 static void
 finalize (GObject *object)
 {
-	SoupConnectionNTLM *ntlm = SOUP_CONNECTION_NTLM (object);
+	SoupConnectionNTLMPrivate *priv = SOUP_CONNECTION_NTLM_GET_PRIVATE (object);
 
-	g_free (ntlm->priv->user);
-	memset (ntlm->priv->nt_hash, 0, sizeof (ntlm->priv->nt_hash));
-	memset (ntlm->priv->lm_hash, 0, sizeof (ntlm->priv->lm_hash));
+	memset (priv->nt_hash, 0, sizeof (priv->nt_hash));
+	memset (priv->lm_hash, 0, sizeof (priv->lm_hash));
 
-	g_free (ntlm->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (soup_connection_ntlm_parent_class)->finalize (object);
 }
 
 static void
-class_init (GObjectClass *object_class)
+soup_connection_ntlm_class_init (SoupConnectionNTLMClass *connection_ntlm_class)
 {
-	SoupConnectionClass *connection_class =
-		SOUP_CONNECTION_CLASS (object_class);
+	SoupConnectionClass *connection_class = SOUP_CONNECTION_CLASS (connection_ntlm_class);
+	GObjectClass *object_class = G_OBJECT_CLASS (connection_ntlm_class);
 
-	parent_class = g_type_class_ref (PARENT_TYPE);
+	g_type_class_add_private (connection_ntlm_class, sizeof (SoupConnectionNTLMPrivate));
 
 	connection_class->send_request = send_request;
 	object_class->finalize = finalize;
 }
 
-SOUP_MAKE_TYPE (soup_connection_ntlm, SoupConnectionNTLM, class_init, init, PARENT_TYPE)
-
 
 static void
 ntlm_authorize_pre (SoupMessage *msg, gpointer user_data)
 {
 	SoupConnectionNTLM *ntlm = user_data;
+	SoupConnectionNTLMPrivate *priv = SOUP_CONNECTION_NTLM_GET_PRIVATE (ntlm);
 	const GSList *headers;
 	const char *val;
 	char *nonce, *header;
 	char *username, *domain_username = NULL, *password = NULL;
 	char *slash, *domain;
 
-	if (ntlm->priv->state > SOUP_CONNECTION_NTLM_SENT_REQUEST) {
+	if (priv->state > SOUP_CONNECTION_NTLM_SENT_REQUEST) {
 		/* We already authenticated, but then got another 401.
 		 * That means "permission denied", so don't try to
 		 * authenticate again.
 		 */
-		ntlm->priv->state = SOUP_CONNECTION_NTLM_FAILED;
+		priv->state = SOUP_CONNECTION_NTLM_FAILED;
 		goto done;
 	}
 
@@ -111,12 +103,12 @@ ntlm_authorize_pre (SoupMessage *msg, gpointer user_data)
 		headers = headers->next;
 	}
 	if (!headers) {
-		ntlm->priv->state = SOUP_CONNECTION_NTLM_FAILED;
+		priv->state = SOUP_CONNECTION_NTLM_FAILED;
 		goto done;
 	}
 
 	if (!soup_ntlm_parse_challenge (val, &nonce, &domain)) {
-		ntlm->priv->state = SOUP_CONNECTION_NTLM_FAILED;
+		priv->state = SOUP_CONNECTION_NTLM_FAILED;
 		goto done;
 	}
 
@@ -132,8 +124,10 @@ ntlm_authorize_pre (SoupMessage *msg, gpointer user_data)
 	slash = strpbrk (domain_username, "\\/");
 	if (slash) {
 		g_free (domain);
-		domain = g_strdup (domain_username);
+		*slash = '\0';
+		domain = domain_username;
 		username = slash + 1;
+		domain_username = NULL;
 	} else
 		username = domain_username;
 
@@ -147,7 +141,7 @@ ntlm_authorize_pre (SoupMessage *msg, gpointer user_data)
 	soup_message_add_header (msg->request_headers,
 				 "Authorization", header);
 	g_free (header);
-	ntlm->priv->state = SOUP_CONNECTION_NTLM_RECEIVED_CHALLENGE;
+	priv->state = SOUP_CONNECTION_NTLM_RECEIVED_CHALLENGE;
 
  done:
 	/* Remove the WWW-Authenticate headers so the session won't try
@@ -159,37 +153,45 @@ ntlm_authorize_pre (SoupMessage *msg, gpointer user_data)
 static void
 ntlm_authorize_post (SoupMessage *msg, gpointer conn)
 {
-	SoupConnectionNTLM *ntlm = conn;
+	SoupConnectionNTLMPrivate *priv = SOUP_CONNECTION_NTLM_GET_PRIVATE (conn);
 
-	if (ntlm->priv->state == SOUP_CONNECTION_NTLM_RECEIVED_CHALLENGE &&
+	if (priv->state == SOUP_CONNECTION_NTLM_RECEIVED_CHALLENGE &&
 	    soup_message_get_header (msg->request_headers, "Authorization")) {
 		/* We just added the last Auth header, so restart it. */
-		ntlm->priv->state = SOUP_CONNECTION_NTLM_SENT_RESPONSE;
+		priv->state = SOUP_CONNECTION_NTLM_SENT_RESPONSE;
+
+		/* soup_message_restarted() will call soup_message_io_stop(),
+		 * which will release the connection, and may cause another
+		 * message to be queued on the connection before it returns.
+		 * That's no good, so we stop the message first and then
+		 * reclaim the connection so that soup_message_restarted()
+		 * won't be able to steal it.
+		 */
+		soup_message_io_stop (msg);
+		soup_connection_reserve (conn);
 		soup_message_restarted (msg);
 		soup_connection_send_request (conn, msg);
 	}
 }
 
 static void
-ntlm_cleanup_msg (SoupMessage *msg, gpointer user_data)
+ntlm_cleanup_msg (SoupMessage *msg, gpointer conn)
 {
-	SoupConnectionNTLM *ntlm = user_data;
-
 	/* Do this when the message is restarted, in case it's
 	 * restarted on a different connection.
 	 */
 	soup_message_remove_handler (msg, SOUP_HANDLER_PRE_BODY,
-				     ntlm_authorize_pre, ntlm);
+				     ntlm_authorize_pre, conn);
 	soup_message_remove_handler (msg, SOUP_HANDLER_POST_BODY,
-				     ntlm_authorize_post, ntlm);
+				     ntlm_authorize_post, conn);
 }
 
 void
 send_request (SoupConnection *conn, SoupMessage *req)
 {
-	SoupConnectionNTLM *ntlm = SOUP_CONNECTION_NTLM (conn);
+	SoupConnectionNTLMPrivate *priv = SOUP_CONNECTION_NTLM_GET_PRIVATE (conn);
 
-	if (ntlm->priv->state == SOUP_CONNECTION_NTLM_NEW) {
+	if (priv->state == SOUP_CONNECTION_NTLM_NEW) {
 		char *header = soup_ntlm_request ();
 
 		soup_message_remove_header (req->request_headers,
@@ -197,7 +199,7 @@ send_request (SoupConnection *conn, SoupMessage *req)
 		soup_message_add_header (req->request_headers,
 					 "Authorization", header);
 		g_free (header);
-		ntlm->priv->state = SOUP_CONNECTION_NTLM_SENT_REQUEST;
+		priv->state = SOUP_CONNECTION_NTLM_SENT_REQUEST;
 	}
 
 	soup_message_add_status_code_handler (req, SOUP_STATUS_UNAUTHORIZED,
@@ -209,11 +211,11 @@ send_request (SoupConnection *conn, SoupMessage *req)
 					      ntlm_authorize_post, conn);
 
 	g_signal_connect (req, "restarted",
-			  G_CALLBACK (ntlm_cleanup_msg), ntlm);
+			  G_CALLBACK (ntlm_cleanup_msg), conn);
 	g_signal_connect (req, "finished",
-			  G_CALLBACK (ntlm_cleanup_msg), ntlm);
+			  G_CALLBACK (ntlm_cleanup_msg), conn);
 
-	SOUP_CONNECTION_CLASS (parent_class)->send_request (conn, req);
+	SOUP_CONNECTION_CLASS (soup_connection_ntlm_parent_class)->send_request (conn, req);
 }
 
 
@@ -224,7 +226,7 @@ static void md4sum                (const unsigned char *in,
 				   unsigned char        digest[16]);
 
 /* DES */
-typedef unsigned long DES_KS[16][2]; /* Single-key DES key schedule */
+typedef guint32 DES_KS[16][2]; /* Single-key DES key schedule */
 
 static void deskey                (DES_KS, unsigned char *, int);
 
@@ -628,7 +630,7 @@ md4sum (const unsigned char *in, int nbytes, unsigned char digest[16])
 
 
 /* Public domain DES implementation from Phil Karn */
-static unsigned long Spbox[8][64] = {
+static guint32 Spbox[8][64] = {
 	{ 0x01010400,0x00000000,0x00010000,0x01010404,
 	  0x01010004,0x00010404,0x00000004,0x00010000,
 	  0x00000400,0x01010400,0x01010404,0x00000400,
@@ -775,20 +777,20 @@ static unsigned long Spbox[8][64] = {
 /* Encrypt or decrypt a block of data in ECB mode */
 static void
 des(ks,block)
-unsigned long ks[16][2];	/* Key schedule */
+guint32 ks[16][2];	/* Key schedule */
 unsigned char block[8];		/* Data block */
 {
-	unsigned long left,right,work;
+	guint32 left,right,work;
 	
 	/* Read input block and place in left/right in big-endian order */
-	left = ((unsigned long)block[0] << 24)
-	 | ((unsigned long)block[1] << 16)
-	 | ((unsigned long)block[2] << 8)
-	 | (unsigned long)block[3];
-	right = ((unsigned long)block[4] << 24)
-	 | ((unsigned long)block[5] << 16)
-	 | ((unsigned long)block[6] << 8)
-	 | (unsigned long)block[7];
+	left = ((guint32)block[0] << 24)
+	 | ((guint32)block[1] << 16)
+	 | ((guint32)block[2] << 8)
+	 | (guint32)block[3];
+	right = ((guint32)block[4] << 24)
+	 | ((guint32)block[5] << 16)
+	 | ((guint32)block[6] << 8)
+	 | (guint32)block[7];
 
 	/* Hoey's clever initial permutation algorithm, from Outerbridge
 	 * (see Schneier p 478)	
@@ -943,13 +945,13 @@ int decrypt;			/* 0 = encrypt, 1 = decrypt */
 			}
 		}
 		/* Now convert to packed odd/even interleaved form */
-		k[i][0] = ((long)ks[0] << 24)
-		 | ((long)ks[2] << 16)
-		 | ((long)ks[4] << 8)
-		 | ((long)ks[6]);
-		k[i][1] = ((long)ks[1] << 24)
-		 | ((long)ks[3] << 16)
-		 | ((long)ks[5] << 8)
-		 | ((long)ks[7]);
+		k[i][0] = ((guint32)ks[0] << 24)
+		 | ((guint32)ks[2] << 16)
+		 | ((guint32)ks[4] << 8)
+		 | ((guint32)ks[6]);
+		k[i][1] = ((guint32)ks[1] << 24)
+		 | ((guint32)ks[3] << 16)
+		 | ((guint32)ks[5] << 8)
+		 | ((guint32)ks[7]);
 	}
 }
